/*
    FEARLUS model0-5-1-5: ObserverSwarm.m
    Copyright (C) 1999-2004  Macaulay Institute

    This file is part of FEARLUS model0-5-1-5, an agent-based model of land use
    change.

    FEARLUS model0-5-1-5 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-5-1-5 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*


Implementation for the ObserverSwarm class. As per the header, this is based
heavily on the tutorial material.

*/

#import "ObserverSwarm.h"
#import "ModelSwarm.h"
#import "Environment.h"
#import "LandAllocator.h"
#import "EZBar.h"
#import "DBColormap.h"
#import "FearlusArguments.h"
#import "Parameter.h"
#import "LandUse.h"
#import "LandParcel.h"
#import "MiscFunc.h"		// getUsableFileNameFrom:withSuffix:
#import "SubPopulation.h"
#import "Reporter.h"
#import "StrategyColourKey.h"

#define SOLVED_AVERAGE_BUG

@implementation ObserverSwarm

/*

createBegin

Create an ObserverSwarm object. Set up the probes for the class.

*/

+createBegin: (id)z {
  ObserverSwarm *o;
  id <ProbeMap> probeMap;

  o = [super createBegin: z];
  o->displayFrequency = 1;
  o->writeHistoryFrequency = 500;
  o->dumpHistory = YES;
  o->saveDumpedHistory = NO;
  o->historyFileName = (char *)malloc(1024);
  sprintf(o->historyFileName, [arguments getHistoryFile]);
  o->showStrategyColourKey = 1;
  o->showUseRaster = 1;
  o->showManagerRaster = 1;
  o->showStrategyRaster = 1;
  o->showUseChangeRaster = 1;
  o->showManagerChangeRaster = 1;
  o->showParcelsOwnedGraph = 1;
  o->showYieldGraph = 1;
  o->showPopulationGraph = 1;
  o->showUseGraph = 1;
  o->showWealthGraph = 1;
  o->showAgeGraph = 1;
  o->showNWeightDist = 1;
  o->showThresholdDist = 1;
  o->showImitProbDist = 1;
  o->nHistogramBins = 8;
  o->showUseBar = 0;

  probeMap = [EmptyProbeMap createBegin: z];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];

  [probeMap addProbe: [probeLibrary 
			getProbeForVariable: "showStrategyColourKey"
			inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "showUseRaster"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "showManagerRaster"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "showStrategyRaster"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "showUseChangeRaster"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
			getProbeForVariable: "showManagerChangeRaster"
			inClass: [self class]]];
  [probeMap addProbe: [probeLibrary
			getProbeForVariable: "showParcelsOwnedGraph"
			inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "showYieldGraph"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "showPopulationGraph"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "showUseGraph"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "showWealthGraph"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "showAgeGraph"
				    inClass: [self class]]];
  /*
  [probeMap addProbe: [probeLibrary getProbeForVariable: "showUseBar"
				    inClass: [self class]]];
  */
  [probeMap addProbe: [probeLibrary getProbeForVariable: "showNWeightDist"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "showThresholdDist"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "showImitProbDist"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "nHistogramBins"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "displayFrequency"
				    inClass: [self class]]];
  [probeMap addProbe:
	      [probeLibrary getProbeForVariable: "writeHistoryFrequency"
			    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "dumpHistory"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "saveDumpedHistory"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForVariable: "historyFileName"
				    inClass: [self class]]];
  [probeMap addProbe: [probeLibrary getProbeForMessage: "writeHistory"
				    inClass: [self class]]];
  // I might want to set up probes for the suggestion in the header file --
  // i.e. have some flags or options for which displays to use.
  [probeLibrary setProbeMap: probeMap For: [self class]];
  return(o);
}

/*

createEnd

I'm sure this over-riding of the super class createEnd is utterly pointless,
since it doesn't do anything else

*/

-createEnd {
  return([super createEnd]);
}

/*

buildGUIObjects

Build the (non-Swarm standard) GUI objects

*/

-(void)buildGUIObjects {
  //  [globalTkInterp eval:
  //		    "tk_getOpenFile -filetypes {{{LandParcel Files} {.lp}}}"];
  //  printf("%s\n", [globalTkInterp result]);
}

/*

buildReporter

Build the reporter object, and its schedule

*/

-buildReporter {
  reporter = [Reporter createBegin: self];
  [reporter setParameters: parameter];
  [reporter setModelSwarm: modelSwarm];
  reporter = [reporter createEnd];
  reporterSchedule = [Schedule createBegin: self];
  [reporterSchedule setRepeatInterval: 1];
  reporterSchedule = [reporterSchedule createEnd];
  [reporterSchedule at: 0 createActionTo: reporter message: M(report)];
  [reporterSchedule activateIn: self];

  return self;
}

/*

buildObjects

Build the objects needed for the class

*/

-buildObjects {
  struct stat modelfile;
  double r, g, b;
  int i, initZoom, maxEnvDim;

  [super buildObjects];
  // Load in any observer parameters if specified
  if([arguments observerFileHasBeenSpecified]) {
    [ObjectLoader load: self fromFileNamed: [arguments getObserverFile]];
  }
  // Build the parameter object
  parameter = [Parameter createBegin: self];
  if([MiscFunc fileExists: [arguments getModelFile]]) {
				// Test to see if a model file exists before
                                // loading it.
    if([arguments debugObserver]) {
      printf("o: Loading model parameters from file %s\n",
	     [arguments getModelFile]);
    }
    [parameter loadFromFileNamed: [arguments getModelFile]];
    dumpHistory = [parameter dumpHistory];
    writeHistoryFrequency = [parameter maxHistoryLength];
  }
  else {
    fprintf(stderr, "Model parameter file %s not found -- using default values"
	    " for parameters\n", [arguments getModelFile]);
    [parameter setDumpHistory: dumpHistory];
    [parameter setMaxHistoryLength: writeHistoryFrequency];
  }
  if([arguments seedHasBeenSpecified]) {
    [parameter setSpecifiedSeed: seed];
  }
  parameter = [parameter createEnd];


  CREATE_PROBE_DISPLAY(parameter);
  //  CREATE_PROBE_DISPLAY(modelSwarm);
  CREATE_PROBE_DISPLAY(self);

  [self buildGUIObjects];

  [controlPanel setStateStopped];
  if([controlPanel getState] == ControlStateQuit) {
    exit(0);
  }
  // Now I was reading in the library about control panels, I think they are
  // called activity controls, and am I sure that I don't want to use one of
  // those?

  [parameter finalise];
  // Build the model swarm -- this should not happen until the parameter object
  // has been finalised.
  modelSwarm = [ModelSwarm createBegin: self];
  [modelSwarm setParameters: parameter];
  modelSwarm = [modelSwarm createEnd];

  [parameter setDumpHistory: dumpHistory];
  [parameter setMaxHistoryLength: writeHistoryFrequency];
  [modelSwarm buildObjects];

  colorMap = [DBColormap create: self];

  [colorMap setColor: 0 ToName: "Gold"];
  for(i = 1; i < 40; i++) {
    r = g = b = ((double)i * 6.0) / 256.0;
    if(![colorMap setColor: i ToRed: r Green: g Blue: b]) {
				// Colours 1-39 are stepped grey levels, for
				// drawing graphs
      break;
    }
  }
  for(; (i >= 40) && (i < 256); i++) {
                                // Don't run this loop if we haven't allocated
				// all 40 grey levels
    r = (double)((((i - 40) / 36) * 30) + 100) / 256.0;
    g = (double)(((((i - 40) % 36) / 6) * 30) + 100) / 256.0;
    b = (double)((((i - 40) % 6) * 30) + 100) / 256.0;
    if(![colorMap setColor: i ToRed: r Green: g Blue: b]) {
				// Colours 40-255 are in the range 100-250 in
				// steps of 30 for each of red, green and blue
				// This should provide a variety of colours
                                // for labelling different land managers and
				// parcels.
      break;
    }
  }
  if([arguments debugObserver]) {
    printf("o: Allocated %d colours\n", i - 1);
  }
  [[modelSwarm getEnvironment] setMaxColour: i - 1];
  if(i < 40) {
    [[modelSwarm getLandAllocator] setMinColour: 0 max: i - 1];
  }
  else {
    [[modelSwarm getLandAllocator] setMinColour: 40 max: i - 1];
  }
  
  // Create the colour map for the strategy display
  legend = [StrategyColourKey create: self];
  strategyCMap = [legend getColourMap];
  if(showStrategyColourKey) [legend display];
  [parameter setStrategyColourKey: legend];

  // [modelSwarm initialiseObjects];
  // Initialisation of objects is now done through the schedule
				// Now everything that needs to know how many
                                // colours there are knows how many colours
				// there are, they can get creating.

/* In the heatbugs ObserverSwarm, they have this:
   envRaster = [ZoomRaster createBegin: [self getZone]];
   SET_WINDOW_GEOMETRY_RECORD_NAME(envRaster);
   envRaster = [envRaster createEnd];
   [envRaster enableDestroyNotification: self
              notificationMethod: @selector(_envRasterDeath_:)];
	                          // why not M(_envRasterDeath_:) ?
				  // isn't M() a macro for @selector() ?

   _envRasterDeath_ is a method of the ObserverSwarm class that looks like:

-_envRasterDeath_: (id)caller {
  [envRaster drop];
  envRaster = nil;
  return self;
}

   Here's what the documentation set has to say about this:

   WindowGeometryRecordName is a protocol for archiving window geometry.
Classes that allow for window geometry archiving must conform to this protocol.
The macro SET_WINDOW_...(aWidget) uses the instance name of aWidget to set its
name in the window geometry record.

   The Raster protocol adopts the ArchivedGeometryWidget protocol, which
provides an interface to destroy notification. The enableDestroyNotification...
method, with arguments targetObject and targetObjectMethod enables a
notification to be sent to the targetObjectMethod if the envRaster is
destroyed.

   So that's all clear then.

   So what happens in buildActions in heatbugs? The action in displayActions
   just sends a message to the observerSwarm to run the _update_ method. The
   _update_ method goes through each displayed object in turn, and if it
   is not nil, calls the methods to display.
   */

  // Decide the initial zoom factor, on the basis of the larger of the
  // environment's dimensions. Aim for a raster of about 250 x 250 pixels.

  maxEnvDim = [[modelSwarm getEnvironment] getSizeX];
  if([[modelSwarm getEnvironment] getSizeY] > maxEnvDim) {
    maxEnvDim = [[modelSwarm getEnvironment] getSizeY];
  }
  initZoom = 250 / maxEnvDim;
  
  // Create the raster for the land use display

  if(showUseRaster) {
    envRaster = [ZoomRaster create: self];
    [envRaster setColormap: [colorMap getColormap]];
    [envRaster setZoomFactor: initZoom];
    [envRaster setWidth: [[modelSwarm getEnvironment] getSizeX]
	       Height: [[modelSwarm getEnvironment] getSizeY]];
    [envRaster setWindowTitle: "Land Parcels by Land Use"];
    [envRaster pack];		// This apparently means draw it
  }

  // Create the raster for the land use change display

  if(showUseChangeRaster) {
    landUseChangeRaster = [ZoomRaster create: self];
    [landUseChangeRaster setColormap: [colorMap getColormap]];
    [landUseChangeRaster setZoomFactor: initZoom];
    [landUseChangeRaster setWidth: [[modelSwarm getEnvironment] getSizeX]
			 Height: [[modelSwarm getEnvironment] getSizeY]];
    [landUseChangeRaster setWindowTitle:
			   "Land Parcels by Land Use Change"];
    [landUseChangeRaster pack];
  }

  // Create the raster for the land manager display

  if(showManagerRaster) {
    landMgrRaster = [ZoomRaster create: self];
    [landMgrRaster setColormap: [colorMap getColormap]];
    [landMgrRaster setZoomFactor: initZoom];
    [landMgrRaster setWidth: [[modelSwarm getEnvironment] getSizeX]
		   Height: [[modelSwarm getEnvironment] getSizeY]];
    [landMgrRaster setWindowTitle: "Land Parcels by Land Manager"];
    [landMgrRaster pack];
  }

  // Create the raster for the land manager change display

  if(showManagerChangeRaster) {
    landMgrChangeRaster = [ZoomRaster create: self];
    [landMgrChangeRaster setColormap: [colorMap getColormap]];
    [landMgrChangeRaster setZoomFactor: initZoom];
    [landMgrChangeRaster setWidth: [[modelSwarm getEnvironment] getSizeX]
			 Height: [[modelSwarm getEnvironment] getSizeY]];
    [landMgrChangeRaster 
      setWindowTitle: "Land Parcels by Land Manager Change"];
    [landMgrChangeRaster pack];
  }

  // Create the raster for the strategy display

  if(showStrategyRaster) {
    strategyRaster = [ZoomRaster create: self];
    [strategyRaster setColormap: strategyCMap];
    [strategyRaster setZoomFactor: initZoom];
    [strategyRaster setWidth: [[modelSwarm getEnvironment] getSizeX]
		    Height: [[modelSwarm getEnvironment] getSizeY]];
    [strategyRaster setWindowTitle: "Parcels by Strategy used to select Use"];
    [strategyRaster pack];
  }

  // Create the land use display

  if(showUseRaster) {
    landParcelDisplay = [Object2dDisplay createBegin: self];
    [landParcelDisplay setDisplayWidget: envRaster];
    [landParcelDisplay setDiscrete2dToDisplay: [modelSwarm getEnvironment]];
    [landParcelDisplay setObjectCollection:
			 [[modelSwarm getEnvironment] getLandParcels]];
    [landParcelDisplay setDisplayMessage: M(drawSelfOn:)];
    landParcelDisplay = [landParcelDisplay createEnd];

    parcelBoundaryDisplay = [Object2dDisplay createBegin: self];
    [parcelBoundaryDisplay setDisplayWidget: envRaster];
    [parcelBoundaryDisplay setDiscrete2dToDisplay:
			     [modelSwarm getEnvironment]];
    [parcelBoundaryDisplay setObjectCollection:
			     [[modelSwarm getEnvironment] getLandParcels]];
    [parcelBoundaryDisplay setDisplayMessage: M(drawMatchLinesOn:)];
    parcelBoundaryDisplay = [parcelBoundaryDisplay createEnd];

    parcelNbrDisplay = [Object2dDisplay createBegin: self];
    [parcelNbrDisplay setDisplayWidget: envRaster];
    [parcelNbrDisplay setDiscrete2dToDisplay: [modelSwarm getEnvironment]];
    [parcelNbrDisplay setObjectCollection:
			[[modelSwarm getEnvironment] getLandParcels]];
    [parcelNbrDisplay setDisplayMessage: M(drawNbrLinesOn:)];
    parcelNbrDisplay = [parcelNbrDisplay createEnd];
  }

  // Create the land use change display

  if(showUseChangeRaster) {
    landUseChangeDisplay = [Object2dDisplay createBegin: self];
    [landUseChangeDisplay setDisplayWidget: landUseChangeRaster];
    [landUseChangeDisplay setDiscrete2dToDisplay: [modelSwarm getEnvironment]];
    [landUseChangeDisplay setObjectCollection:
			    [[modelSwarm getEnvironment] getLandParcels]];
    [landUseChangeDisplay setDisplayMessage: M(drawChangeLURankOn:)];
    landUseChangeDisplay = [landUseChangeDisplay createEnd];
  }

  // Create the land manager display

  if(showManagerRaster) {
    landMgrDisplay = [Object2dDisplay createBegin: self];
    [landMgrDisplay setDisplayWidget: landMgrRaster];
    [landMgrDisplay setDiscrete2dToDisplay: [modelSwarm getEnvironment]];
    [landMgrDisplay setObjectCollection:
		      [[modelSwarm getEnvironment] getLandParcels]];
    [landMgrDisplay setDisplayMessage: M(drawMgrOn:)];
    landMgrDisplay = [landMgrDisplay createEnd];
  }

  // Create the land manager change display

  if(showManagerChangeRaster) {
    landMgrChangeDisplay = [Object2dDisplay createBegin: self];
    [landMgrChangeDisplay setDisplayWidget: landMgrChangeRaster];
    [landMgrChangeDisplay setDiscrete2dToDisplay: [modelSwarm getEnvironment]];
    [landMgrChangeDisplay setObjectCollection:
			    [[modelSwarm getEnvironment] getLandParcels]];
    [landMgrChangeDisplay setDisplayMessage: M(drawChangeLMgrRankOn:)];
    landMgrChangeDisplay = [landMgrChangeDisplay createEnd];
  }

  // Create the strategy display

  if(showStrategyRaster) {
    strategyDisplay = [Object2dDisplay createBegin: self];
    [strategyDisplay setDisplayWidget: strategyRaster];
    [strategyDisplay setDiscrete2dToDisplay: [modelSwarm getEnvironment]];
    [strategyDisplay setObjectCollection:
		       [[modelSwarm getEnvironment] getLandParcels]];
    [strategyDisplay setDisplayMessage: M(drawStrategyOn:)];
    strategyDisplay = [strategyDisplay createEnd];
  }

  // Enable right button click on each of the rasters

  if(showUseRaster) {
    [envRaster setButton: ButtonRight Client: landParcelDisplay
	       Message: M(makeProbeAtX:Y:)];
  }
  if(showUseChangeRaster) {
    [landUseChangeRaster setButton: ButtonRight Client: landUseChangeDisplay
			 Message: M(makeProbeAtX:Y:)];
  }
  if(showManagerRaster) {
    [landMgrRaster setButton: ButtonRight Client: landMgrDisplay
		   Message: M(makeProbeAtX:Y:)];
  }
  if(showManagerChangeRaster) {
    [landMgrChangeRaster setButton: ButtonRight Client: landMgrChangeDisplay
			 Message: M(makeProbeAtX:Y:)];
  }
  if(showStrategyRaster) {
    [strategyRaster setButton: ButtonRight Client: strategyDisplay
		    Message: M(makeProbeAtX:Y:)];
  }

  // Enable left button click on some of the rasters

  // On the land manager raster, enable the user to display the physical and
  // social neighbourhood of a particular land manager
  if(showManagerRaster) {
    [landMgrRaster setButton: ButtonLeft Client: self
		   Message: M(toggleMgrNeighbourhoodOfX:Y:)];
  }

  // On the land parcel raster, enable the user to display the physical
  // neighbourhood of a particular cell
  if(showUseRaster) {
    [envRaster setButton: ButtonLeft Client: self
	       Message: M(togglePhysNeighbourhoodOfX:Y:)];
  }
  

  // Create the land manager land parcel graph

  if(showParcelsOwnedGraph) {
    parcelsOwnedGraph = [EZGraph createBegin: [self getZone]];
    [parcelsOwnedGraph setTitle:
			 "Land Parcels Owned by Land Managers"];
    [parcelsOwnedGraph setAxisLabelsX: "time" Y: "number of land parcels"];
    parcelsOwnedGraph = [parcelsOwnedGraph createEnd];
  }

  // Create the land manager wealth graph

  if(showWealthGraph) {
    wealthGraph = [EZGraph createBegin: [self getZone]];
    [wealthGraph setTitle: "Wealth of Land Managers vs. time"];
    [wealthGraph setAxisLabelsX: "time" Y: "wealth"];
    wealthGraph = [wealthGraph createEnd];
  }

  // Create the land manager age graph

  if(showAgeGraph) {
    ageGraph = [EZGraph createBegin: [self getZone]];
    [ageGraph setTitle: "Age of Land Managers vs. time"];
    [ageGraph setAxisLabelsX: "time" Y: "age"];
    ageGraph = [ageGraph createEnd];
  }

  // Create the yield graph

  if(showYieldGraph) {
    yieldGraph = [EZGraph createBegin: [self getZone]];
    [yieldGraph setTitle: "Yield of Land Parcels vs. time"];
    [yieldGraph setAxisLabelsX: "time" Y: "yield"];
    yieldGraph = [yieldGraph createEnd];
  }

  // Create the population graph

  if(showPopulationGraph) {
    landMgrPopGraph = [EZGraph createBegin: [self getZone]];
    [landMgrPopGraph setTitle: "Population of Land Managers vs. time"];
    [landMgrPopGraph setAxisLabelsX: "time" Y: "population"];
    landMgrPopGraph = [landMgrPopGraph createEnd];
  }

  // Create the land use graph

  if(showUseGraph) {
    landUseGraph = [EZGraph createBegin: [self getZone]];
    [landUseGraph setTitle: "Number of Land Parcels for each Land Use"];
    [landUseGraph setAxisLabelsX: "time" Y: "parcels"];
    landUseGraph = [landUseGraph createEnd];
  }

  // Create the arrays for the various distribution graphs

  if(showNWeightDist) {
    nbrwgtDists = [List create: self];
  }
  if(showThresholdDist) {
    happyThreshDists = [List create: self];
  }
  if(showImitProbDist) {
    imitProbDists = [List create: self];
  }
  // We'll be adding the sequences later (once the land uses have been created)

  // Create the land use bar chart
  /*
  landUseBar = [EZBar createBegin: [self getZone]];
  [landUseBar setTitle: "Average Number of Land Parcels for each Land Use"];
  [landUseBar setAxisLabelsX: "Land Uses" Y: "Number of Land Parcels"];
  [landUseBar setCollection: [[modelSwarm getEnvironment] getLandUses]];
  [landUseBar setProbedSelector: M(getNLandParcels)];
//  [landUseBar setLabelSelector: M(getLabel)];
  [landUseBar setColourSelector: M(getColour) andColourmap: colorMap];
  landUseBar = [landUseBar createEnd];

Can't do this just now. The initialisation stage creates the land uses, which
must be called to get their colour. The thing is, the initialisation stage
hasn't taken place yet. Damn.

  */

  // Don't forget to add any new graphical display to the schedule in
  // buildActions

  return(self);
}

/*

toggleMgrNeighbourhoodOfX:Y:

Call the LandParcel at X, Y and tell it to display/undisplay the physical and
social neighbourhoods of the land manager owning the land parcel at X, Y.

*/

-toggleMgrNeighbourhoodOfX: (int)x Y: (int)y {
  [(LandParcel *)[[modelSwarm getEnvironment] 
		   getObjectAtX: x Y: y] toggleMgrNbrDisplay];
  return self;
}

/*

togglePhysNeighbourhoodOfX:Y:

Call the LandParcel at X, Y and tell it to display/undisplay its physical 
neighbourhood.

*/

-togglePhysNeighbourhoodOfX: (int)x Y: (int)y {
  [(LandParcel *)[[modelSwarm getEnvironment]
		   getObjectAtX: x Y: y] togglePhysNbrDisplay];
  return self;
}

/*

populateGraphs

Do any actions for the graphs that require things to have been created.

*/

-(void)populateGraphs {
  int i;
  id landUses;

  // Populate the land use graph
  if(showUseGraph) {
    landUses = [[modelSwarm getEnvironment] getLandUses];
    for(i = 0; i < [landUses getCount]; i++) {
      [landUseGraph createSequence: [[landUses atOffset: i] getLabel]
		    withFeedFrom:	[landUses atOffset: i]
		    andSelector: M(getNLandParcels)];
    }
  }
  // Populate the parcels owned graph
  if([parameter nSubPopulations] == 1) {
    if(showParcelsOwnedGraph) {
      [parcelsOwnedGraph createAverageSequence: "average"
			 withFeedFrom:
			   [[modelSwarm getLandAllocator] getLandManagers]
			 andSelector: M(getNumberOfLandParcelsOwned)];
      [parcelsOwnedGraph createMaxSequence: "maximum"
			 withFeedFrom:
			   [[modelSwarm getLandAllocator] getLandManagers]
			 andSelector: M(getNumberOfLandParcelsOwned)];
      [parcelsOwnedGraph createMinSequence: "minimum"
			 withFeedFrom:
			   [[modelSwarm getLandAllocator] getLandManagers]
			 andSelector: M(getNumberOfLandParcelsOwned)];
    }
    // Populate the wealth graph
    if(showWealthGraph) {
      [wealthGraph createAverageSequence: "average"
		   withFeedFrom: 
		     [[modelSwarm getLandAllocator] getLandManagers]
		   andSelector: M(getBarnAsDouble)];
      [wealthGraph createMaxSequence: "maximum"
		   withFeedFrom: 
		     [[modelSwarm getLandAllocator] getLandManagers]
		   andSelector: M(getBarnAsDouble)];
      [wealthGraph createMinSequence: "minimum"
		   withFeedFrom:
		     [[modelSwarm getLandAllocator] getLandManagers]
		   andSelector: M(getBarnAsDouble)];
    }
    // Populate the age graph
    if(showAgeGraph) {
      [ageGraph createAverageSequence: "average"
		withFeedFrom: [[modelSwarm getLandAllocator] getLandManagers]
		andSelector: M(getAgeAsInt)];
      [ageGraph createMaxSequence: "maximum"
		withFeedFrom: [[modelSwarm getLandAllocator] getLandManagers]
		andSelector: M(getAgeAsInt)];
      [ageGraph createMinSequence: "minimum"
		withFeedFrom: [[modelSwarm getLandAllocator] getLandManagers]
		andSelector: M(getAgeAsInt)];
    }
  }
  else {
    for(i = 0; i < [parameter nSubPopulations]; i++) {
      SubPopulation *sp;

      sp = [parameter subPopulation: i];

      if(showParcelsOwnedGraph) {
	[parcelsOwnedGraph createTotalSequence: [sp getLabel]
			   withFeedFrom: [sp getLandManagers]
			   andSelector: M(getNumberOfLandParcelsOwned)];
      }
      if(showWealthGraph) {
	[wealthGraph createAverageSequence: [sp getLabel]
		     withFeedFrom: [sp getLandManagers]
		     andSelector: M(getBarnAsDouble)];
      }
      if(showAgeGraph) {
	[ageGraph createAverageSequence: [sp getLabel]
		  withFeedFrom: [sp getLandManagers]
		  andSelector: M(getAgeAsInt)];
      }
    }
  }
  // Populate the yield graph
  if(showYieldGraph) {
#ifdef SOLVED_AVERAGE_BUG
    [yieldGraph createAverageSequence: "average"
		withFeedFrom: [[modelSwarm getEnvironment] getLandParcels]
		andSelector: M(getIntYield)];
#else
    [yieldGraph createTotalSequence: "total"
		withFeedFrom: [[modelSwarm getEnvironment] getLandParcels]
		andSelector: M(getIntYield)];
#endif
    [yieldGraph createMaxSequence: "maximum"
		withFeedFrom: [[modelSwarm getEnvironment] getLandParcels]
		andSelector: M(getIntYield)];
    [yieldGraph createMinSequence: "minimum"
		withFeedFrom: [[modelSwarm getEnvironment] getLandParcels]
		andSelector: M(getIntYield)];
  }

  // Populate the population graph

  if(showPopulationGraph) {
    for(i = 0; i < [parameter nSubPopulations]; i++) {
      [landMgrPopGraph createSequence: [[parameter subPopulation: i] getLabel]
		       withFeedFrom: [parameter subPopulation: i]
		       andSelector: M(getManagerCount)];
    }
  }

  // Create the neighbourhood weighting histogram(s)

  if(showNWeightDist) {
    id <EZBin> nbrwgtDist;

    for(i = 0; i < [parameter nSubPopulations]; i++) {
      double lower, upper;

      lower = [[parameter subPopulation: i] getNeighbourWeightMin];
      upper = [[parameter subPopulation: i] getNeighbourWeightMax];
      if(lower == upper) {
	fprintf(stderr, "Not creating a neighbourhood weighting distribution "
		"graph for %s because lower bound %g = upper bound %g\n",
		[[parameter subPopulation: i] getLabel], lower, upper);
	continue;
      }
      nbrwgtDist = [EZBin createBegin: self];
      [nbrwgtDist setTitle: [[parameter subPopulation: i] getLabel]];
      [nbrwgtDist setAxisLabelsX: "neighbourhood weighting" Y: "#managers"];
      [nbrwgtDist setMonoColorBars: YES];
      [nbrwgtDist setBinNum: nHistogramBins];
      [nbrwgtDist setLowerBound: lower];
      [nbrwgtDist setUpperBound: upper];
      [nbrwgtDist setCollection:
		    [[parameter subPopulation: i] getLandManagers]];
      [nbrwgtDist setProbedSelector: M(getNeighbourWeight)];
      nbrwgtDist = [nbrwgtDist createEnd];
      [nbrwgtDists addLast: nbrwgtDist];
    }
  }

  // Create the happiness threshold histogram(s)

  if(showThresholdDist) {
    id <EZBin> happyThreshDist;

    for(i = 0; i < [parameter nSubPopulations]; i++) {
      double lower, upper;

      lower = [[parameter subPopulation: i] getHabitThresholdMin];
      upper = [[parameter subPopulation: i] getHabitThresholdMax];
      if(lower == upper) {
	fprintf(stderr, "Not creating a happiness threshold distribution "
		"graph for %s because lower bound %g = upper bound %g\n",
		[[parameter subPopulation: i] getLabel], lower, upper);
	continue;
      }
      happyThreshDist = [EZBin createBegin: self];
      [happyThreshDist setTitle: [[parameter subPopulation: i] getLabel]];
      [happyThreshDist setAxisLabelsX: "happiness threshold"
		       Y: "#managers"];
      [happyThreshDist setMonoColorBars: YES];
      [happyThreshDist setBinNum: nHistogramBins];
      [happyThreshDist setLowerBound: lower];
      [happyThreshDist setUpperBound: upper];
      [happyThreshDist setCollection:
			 [[parameter subPopulation: i] getLandManagers]];
      [happyThreshDist setProbedSelector: M(getHabitThreshold)];
      happyThreshDist = [happyThreshDist createEnd];
      [happyThreshDists addLast: happyThreshDist];
    }
  }

  // Create the imitative probability histogram(s)

  if(showImitProbDist) {
    id <EZBin> imitProbDist;

    for(i = 0; i < [parameter nSubPopulations]; i++) {
      double lower, upper;

      lower = [[parameter subPopulation: i] getImitateProbMin];
      upper = [[parameter subPopulation: i] getImitateProbMax];
      if(lower == upper) {
	fprintf(stderr, "Not creating an imitate probability distribution "
		"graph for %s because lower bound %g = upper bound %g\n",
		[[parameter subPopulation: i] getLabel], lower, upper);
	continue;
      }
      imitProbDist = [EZBin createBegin: self];
      [imitProbDist setTitle: [[parameter subPopulation: i] getLabel]];
      [imitProbDist setAxisLabelsX: "imitate probability"
		       Y: "#managers"];
      [imitProbDist setMonoColorBars: YES];
      [imitProbDist setBinNum: nHistogramBins];
      [imitProbDist setLowerBound: lower];
      [imitProbDist setUpperBound: upper];
      [imitProbDist setCollection:
		    [[parameter subPopulation: i] getLandManagers]];
      [imitProbDist setProbedSelector: M(getImitateProb)];
      imitProbDist = [imitProbDist createEnd];
      [imitProbDists addLast: imitProbDist];
    }
  }
}

/*

buildActions

Over-ride the Swarm buildActions method to introduce the scheduling required
for this class.

*/

-buildActions {
  int i;

  [super buildActions];
  [modelSwarm buildActions];
  displayActions = [ActionGroup create: self];

  [displayActions createActionTo: [modelSwarm getEnvironment]
		  message: M(rankLandParcels)];
  if(showUseRaster) {
    [displayActions
      createActionTo: landParcelDisplay 	message: M(display)];
    [displayActions
      createActionTo: parcelBoundaryDisplay 	message: M(display)];
    [displayActions
      createActionTo: parcelNbrDisplay		message: M(display)];
    [displayActions
      createActionTo: envRaster 		message: M(drawSelf)];
  }
  if(showUseChangeRaster) {
    [displayActions
      createActionTo: landUseChangeDisplay 	message: M(display)];
    [displayActions
      createActionTo: landUseChangeRaster 	message: M(drawSelf)];
  }
  if(showManagerChangeRaster) {
    [displayActions
      createActionTo: landMgrChangeDisplay 	message: M(display)];
    [displayActions
      createActionTo: landMgrChangeRaster 	message: M(drawSelf)];
  }
  if(showManagerRaster) {
    [displayActions
      createActionTo: landMgrDisplay	 	message: M(display)];
    [displayActions
      createActionTo: landMgrRaster 		message: M(drawSelf)];
  }
  if(showStrategyRaster) {
    [displayActions
      createActionTo: strategyDisplay		message: M(display)];
    [displayActions
      createActionTo: strategyRaster	 	message: M(drawSelf)];
  }
  if(showParcelsOwnedGraph) {
    [displayActions
      createActionTo: parcelsOwnedGraph		message: M(step)];
  }
  if(showWealthGraph) {
    [displayActions
      createActionTo: wealthGraph 		message: M(step)];
  }
  if(showAgeGraph) {
    [displayActions
      createActionTo: ageGraph	 		message: M(step)];
  }
  if(showYieldGraph) {
    [displayActions
      createActionTo: yieldGraph 		message: M(step)];
  }
  if(showPopulationGraph) {
    [displayActions
      createActionTo: landMgrPopGraph	 	message: M(step)];
  }
  if(showUseBar) {
    [displayActions
      createActionTo: landUseBar		message: M(step)];
  }
  if(showUseGraph) {
    [displayActions
      createActionTo: landUseGraph		message: M(step)];
  }
  if(showNWeightDist) {
    [displayActions
      createActionForEach: nbrwgtDists		message: M(reset)];
    [displayActions
      createActionForEach: nbrwgtDists		message: M(update)];
    [displayActions
      createActionForEach: nbrwgtDists		message: M(output)];
  }
  if(showThresholdDist) {
    [displayActions
      createActionForEach: happyThreshDists	message: M(reset)];
    [displayActions
      createActionForEach: happyThreshDists	message: M(update)];
    [displayActions
      createActionForEach: happyThreshDists	message: M(output)];
  }
  if(showImitProbDist) {
    [displayActions
      createActionForEach: imitProbDists	message: M(reset)];
    [displayActions
      createActionForEach: imitProbDists	message: M(update)];
    [displayActions
      createActionForEach: imitProbDists	message: M(output)];
  }
  [displayActions createActionTo: probeDisplayManager 	message: M(update)];
				// If we do more displays, they'll have to
				// be added in here
  [displayActions createActionTo: actionCache message: M(doTkEvents)];

  displaySchedule = [Schedule createBegin: self];
  [displaySchedule setRepeatInterval: displayFrequency];
  displaySchedule = [displaySchedule createEnd];
  [displaySchedule at: 0 createAction: displayActions];

  historySchedule = [Schedule createBegin: self];
  [historySchedule setRepeatInterval: writeHistoryFrequency];
  historySchedule = [historySchedule createEnd];
  [historySchedule at: 0 createActionTo: self
		   message: M(writeHistoryToFile:) : (id)historyFileName];


  initialSchedule = [Schedule createBegin: self];
  initialSchedule = [initialSchedule createEnd];
  [initialSchedule at: 0 createActionTo: self message: M(populateGraphs)];
  [initialSchedule at: 0 createActionTo: displaySchedule
		   message: M(activateIn:) : self];
  if([arguments useReporter]) {
    [initialSchedule at: 0 createActionTo: self message: M(buildReporter)];
  }
  if(saveDumpedHistory) {
    [initialSchedule at: writeHistoryFrequency - 1
		     createActionTo: historySchedule
		     message: M(activateIn:) : self];
  }
  if(![parameter infiniteTime]) {
    [initialSchedule at: [parameter maximumYear]
		     createActionTo: controlPanel
		     message: M(setStateStopped)];
  }


  return(self);
}

/*

activateIn

Over-ride the Swarm activateIn method to activate the schedules

*/

-activateIn: (id)swarmContext {
  [super activateIn: swarmContext];
  [modelSwarm activateIn: self];	// model schedule must come first
  // (This means it will get done first, so the display always occurs after
  // the model cycle)
  [initialSchedule activateIn: self];
  return([self getSwarmActivity]);
}

/*

setSpecifiedSeed:

If the user specified a seed on the command line, then it should be recorded
in the history file. So it needs to be passed in from main.

*/

-(void)setSpecifiedSeed: (unsigned)value {
  seed = value;
}

/*

writeHistory

Call the modelSwarm to get it to write its history to a file that has been
created. This should be made available as a button so it can be called at any
time. If this method has a return type of void, it cannot be made available as
a button. (You get an abort when you click it.)

If the user has specified on the command line that they want a history file
created, then clicking the button will have no effect. A full history will be
written when they quit the application.

*/

-writeHistory {
  if([arguments historyFileHasBeenSpecified]) {
    fprintf(stderr, "History will be written when you quit\n");
  }
  else {
    [self writeHistoryToFile: historyFileName];
  }
  return self;
}

/*

writeHistoryToFile

The method that actually writes the history to a file. Empty file names will
not be written to. This method enforces a .txt suffix for XL. If the file 
exists, it will insert the minimum three digit number before the .txt suffix
required to make the filename unique.

*/

-(void)writeHistoryToFile: (const char *)fileName {
  char *realFileName;
  FILE *fp;

  realFileName = [MiscFunc getUsableFileNameFrom: fileName withSuffix: ".txt"];
  if(realFileName == NULL) {
    fprintf(stderr, "%s -- cannot write history because couldn't create "
	    "datafile\n", sel_get_name(_cmd));
    return;
  }
  // Now we actually write the file
  fp = fopen(realFileName, "w");
  if(!fp) {
    fprintf(stderr, "Cannot create datafile %s\n", realFileName);
    perror(realFileName);
  }
  else {
    printf("Saving data to file: %s\n", realFileName);
    [modelSwarm writeHistory: fp];
    fclose(fp);
  }
  free(realFileName);
}

/*

drop

Over-ride the Swarm drop method to make sure all items created by this object
are dropped

*/

-(void)drop {
  [landMgrDisplay drop];
  [landParcelDisplay drop];
  [landMgrChangeDisplay drop];
  [landUseChangeDisplay drop];
  [strategyDisplay drop];
  //  [landUseBar drop];
  [legend drop];
  [landMgrPopGraph drop];
  [yieldGraph drop];
  [wealthGraph drop];
  [ageGraph drop];
  [parcelsOwnedGraph drop];
  [landMgrRaster drop];
  [landMgrChangeRaster drop];
  [landUseChangeRaster drop];
  [envRaster drop];
  [strategyRaster drop];
  [modelSwarm drop];
  free(historyFileName);
  [super drop];
}

@end
