/*
    FEARLUS model0-5-1-5: SubPopulation.h
    Copyright (C) 1999-2004  Macaulay Institute

    This file is part of FEARLUS model0-5-1-5, an agent-based model of land use
    change.

    FEARLUS model0-5-1-5 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-5-1-5 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*


Interface for the SubPopulation object -- actually a set of parameters
describing different decision-making algorithms. Each land manager will belong
to a given sub-population, using that algorithm to make their land use 
decision. Each SubPopulation specifies a range of values for each of the
habit-imitation threshold, the neighbourhood weighting, and the imitative
strategy probability.

In addition, there should be a GUI for entering all these different values,
and for displaying land managers (i.e. each subPopulation should have a
colour). The land allocator, when creating land managers, should select a
SubPopulation at random (according to a given probability), and hand that
SubPopulation to the land manager, who will use it to get their decision-making
algorithm.

For now, the GUI is just a probe map with a message probe for getting up the
strategy selector GUI. The probe map must get displayed when another method is
called, which is achieved when pressing a button on the parameter object. (Eek
this is getting out of hand.)

*/

#import <objectbase/SwarmObject.h>
#import <collections/List.h>
#import <simtools.h>
#import <simtoolsgui.h>
#import <errno.h>
#import <string.h>
#import "Strategy.h"

@class LandManager, Parameter, StrategySelector;

@interface SubPopulation: SwarmObject {
  double probability;	// Probability that this SubPopulation will be used to
                        // create a land manager.
  int colour;		// Index in a colourmap to use for this SubPopulation.
  StrategySelector *strategies;		// Strategy selector for this sub-pop.
  double neighbourWeightMax;	// Maximum land manager neighbourhood weight
  double neighbourWeightMin;	// Minimum land manager neighbourhood weight
  double imitateProbMax;	// Maximum probability of imitative strategy
  double imitateProbMin;	// Minimum probability of imitative strategy
  unsigned habitImitateThresholdMax;
				// Maximum yield threshold above and beyond
                                // which to use habit imitation
  unsigned habitImitateThresholdMin;
				// Minimum yield threshold above and beyond
                                // which to use habit imitation
  unsigned memorySizeMin;	// Minimum size of memory
  unsigned memorySizeMax;	// Maximum size of memory
  char *strategySelectorFile;	// File to load the strategy selector data from
  unsigned pin;
  char *label;
  id <List> managerList;
  BOOL selectorCreated;
}

+create: aZone;
-(unsigned)getPIN;
-(char *)getLabel;		// Must be created and allocated at create time
-(void)setColour: (int)col;
-(int)getColour;
-(double)getANeighbourWeight;
-(double)getNeighbourWeightMin;	// Used by observer
-(double)getNeighbourWeightMax;	// Used by observer
-(double)getAnImitateProb;
-(double)getImitateProbMin;	// Used by observer
-(double)getImitateProbMax;	// Used by observer
-(unsigned)getAHabitImitateThreshold;
-(double)getHabitThresholdMin;	// Used by observer
-(double)getHabitThresholdMax;	// Used by observer
-(unsigned)getAMemorySize;
-(unsigned)getMaxMemorySize;
-(void)addLandManager: (LandManager *)lm;
-(void)removeLandManager: (LandManager *)lm;
-(id <List>)getLandManagers;
-(id <Strategy>)getAnAboveStrategyForLandManager: (LandManager *)lm
				   andParameters: (Parameter *)p;
-(id <NonImitativeStrategy>)
     getABelowNonImitativeStrategyForLandManager: (LandManager *)lm 
				   andParameters: (Parameter *)p;
-(id <ImitativeStrategy>)
     getABelowImitativeStrategyForLandManager: (LandManager *)lm 
                                andParameters: (Parameter *)p;
-(id <NonImitativeStrategy, NonHistoricalStrategy>)
     getAnInitialStrategyForLandManager: (LandManager *)lm 
                          andParameters: (Parameter *)p;
-(double)getProb;
-loadFromFileNamed: (const char *)filename;
                                        // Just use objectloader for this
-saveToFileNamed: (const char *)filename;
                                  	// Again, use objectsaver -- you can
// use a template probeMap to tell it which instance variables to save.
-(void)writeHistory: (FILE *)fp;
-clickMeToGetStrategySelectorGUI;
-(void)displayProbe;
-(void)drop;
-(unsigned)getManagerCount;

@end
