/*
    FEARLUS model0-5-1-5: main.m
    Copyright (C) 1999-2004  Macaulay Institute

    This file is part of FEARLUS model0-5-1-5, an agent-based model of land use
    change.

    FEARLUS model0-5-1-5 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-5-1-5 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*


The main function. This is copied straight out of Heatbugs, and modified for
the FEARLUS objects.

*/

// This library is distributed without any warranty; without even the
// implied warranty of merchantability or fitness for a particular purpose.
// See file LICENSE for details and terms of copying.

#import <simtools.h>     // initSwarm () and swarmGUIMode
#import <simtoolsgui.h>  // GUISwarm
#import <random.h>
#import <objc/objc-api.h> // objc_trace
#import <objc/Protocol.h>
#import "ObserverSwarm.h"
#import "BatchSwarm.h"
#import "FearlusArguments.h"
#import "Strategy.h"
#import "Report.h"
#import "Clumper.h"
#import "ClassInfo.h"

/*

iterate_over_classes

This function iterates over the classes to see what protocols they follow. This
could be used to build lists of classes that follow particular protocols...

*/

void iterate_over_classes(void) {
  union whats_in_here {
    void *enum_state;
    char **bibble;
  } u;
  struct list_of_classes {
    Class theClass;
    struct list_of_classes *next;
  } *l, *m, *n;
  Class aClass;
  struct objc_protocol_list *pl;

  n = l = NULL;
  while((aClass = objc_next_class(&u.enum_state)) != Nil) {
    m = n;
    n = (struct list_of_classes *)xmalloc(sizeof(struct list_of_classes));
    if(l == NULL) l = n;
    if(n == NULL) {
      fprintf(stderr, "Memory allocation error\n");
      exit(1);
    }
    n->theClass = aClass;
    n->next = NULL;
    if(m != NULL) m->next = n;
  }

  m = NULL;
  for(n = l; n != NULL; n = n->next) {
    BOOL isAStrategy = NO;
    BOOL isAReport = NO;
    BOOL isAClumper = NO;

    if(m != NULL) free(m);
    for(pl = n->theClass->protocols; pl != NULL; pl = pl->next) {
      int i;

      for(i = 0; i < pl->count; i++) {
	if([pl->list[i] conformsTo: @protocol(Strategy)]) isAStrategy = YES;
	if([pl->list[i] conformsTo: @protocol(Report)]) isAReport = YES;
	if([pl->list[i] conformsTo: @protocol(Clumper)]) isAClumper = YES;
      }
    }
    if(isAStrategy) printf("<Strategy> ");
    if(isAClumper) printf("<Clumper> ");
    if(isAReport) printf("<Report> ");
    if(isAStrategy || isAClumper || isAReport) {
      printf("%s\n", class_get_class_name(n->theClass));
    }
    m = n;
  }
  if(m != NULL) free(m);
}

// The main() function is the top-level place where everything starts.
// For a typical Swarm simulation, in main() you create a toplevel
// Swarm, let it build and activate, and set it to running.

int
main (int argc, const char **argv)
{
  id theTopLevelSwarm;
  unsigned seed;

  //  iterate_over_classes();
  //objc_trace = YES;
  /* There's a variable in objc/objc-api.h defined as an extern BOOL called
     objc_trace, that looks (from the comments therein) like it will print out
     the messages as they are called if set to YES. However, I've tried
     compiling this, and it doesn't work. */
  // Swarm initialization: all Swarm apps must call this first.
  initSwarmArguments (argc, argv, [FearlusArguments class]);

  [ClassInfo initialise: [Zone create: globalZone]];

  if([arguments seedHasBeenSpecified]) {
    seed = [arguments getSeedArg];
    printf("Using seed: %u\n", seed);
    [randomGenerator setStateFromSeed: seed];
  }
  else {
    printf("Using default Swarm seed -- cannot get value\n");
  }

  // swarmGUIMode is set in initSwarm(). It's set to be 0 if you
  // typed heatbugs -batchmode. Otherwise, it's set to 1.
  
  if (swarmGUIMode == 1) {
    // We've got graphics, so make a full ObserverSwarm to get GUI objects

    theTopLevelSwarm = [ObserverSwarm createBegin: globalZone];
    SET_WINDOW_GEOMETRY_RECORD_NAME (theTopLevelSwarm);
    theTopLevelSwarm = [theTopLevelSwarm createEnd];
  }
  else {
    // No graphics - make a batchmode swarm and run it.

    theTopLevelSwarm = [BatchSwarm create: globalZone];
  }

  if([arguments seedHasBeenSpecified]) {
    [theTopLevelSwarm setSpecifiedSeed: seed];
  }
  [theTopLevelSwarm buildObjects];
  [theTopLevelSwarm buildActions];
  [theTopLevelSwarm activateIn: nil];
  [theTopLevelSwarm go];
  if([arguments historyFileHasBeenSpecified]) {
    [theTopLevelSwarm writeHistoryToFile: [arguments getHistoryFile]];
  }
//  [theTopLevelSwarm drop]; // This causes a seg fault when dropping EZBar

  // theTopLevelSwarm has finished processing, so it's time to quit.
  return 0;
}
